#include "huc.h"


int byte;
char bit;
char value[12];

main()
{
   set_color(0,0);
   set_color(1,511);

   set_xres(320);

   init_vars();

   while(1) {
      test1();
      vsync(1);
      test2();
      vsync(1);
      test3();
      vsync(1);
   }
}


init_vars()
{
   int i,j;

   for (i=0; i < 4; i++)
   {
      for (j=0; j < 0x10; j++)
      {
         poke((0x1a00 + (i * 0x10) + j), 0);
      }
   }

   for (i=0; i < 0x6; i++)
   {
      poke((0x1ae0 + i), 0);
   }
}


put_binary(in, x, y)
char in;
int x,y;
{
   int i;
   char mask;

   mask = 0x80;	/* bitmask - most significant set; shift later */

   for (i=0; i<8; i++)
   {
      if (in & mask) {
         put_string("1", x+i, y);
      } else {
         put_string("0", x+i, y);
      }
      mask = mask >> 1;
   }
}


/*
 * Test 1 simply displays the Arcade Card identification bytes
 */

test1()
{
   cls();
   put_string("CDROM Arcade Card Test #1", 5, 2);
   put_string("<SEL> for next test", 3, 27);

   put_string("1AFD:", 1, 4);
   put_hex(peek(0x1AFD), 2,  7, 4);
   put_hex(peek(0x1AFE), 2, 10, 4);
   put_hex(peek(0x1AFF), 2, 13, 4);

   while(1)
   {
      if (joytrg(0) & JOY_SLCT)
         break;
   }
}


/*
 * Test 2 is a set of tests to manipulate the shift register
 */

test2()
{
   char shift, shift_neg;

   shift = 1;
   shift_neg = 0xff;
   poke(0x1AE3, 0x5A);

   cls();
   put_string("CDROM Arcade Card Test #2", 5, 2);
   put_string("<UP>/<DN> = change shift amount", 3, 23);
   put_string("<I>       = shift", 3, 24);
   put_string("<II>      = shift negative", 3, 25);
   put_string("<RUN>     = Reinitialize", 3, 26);
   put_string("<SEL> for next test", 3, 27);

   while(1)
   {
      put_string("1AE3:    1AE2:    1AE1:    1AE0:", 1, 4);
      put_binary( peek(0x1AE3), 1, 5);
      put_binary( peek(0x1AE2), 10, 5);
      put_binary( peek(0x1AE1), 19, 5);
      put_binary( peek(0x1AE0), 28, 5);

      put_string("1AE4:", 1, 6);
      put_string("1AE5:", 1, 8);
      put_binary( peek(0x1AE4), 1, 7);
      put_binary( peek(0x1AE5), 1, 9);

      put_string("Shift:",     1, 11);
      put_string("Shift neg:", 1, 12);
      put_hex(shift,     2, 11, 11);
      put_hex(shift_neg, 2, 11, 12);

      /* wait until key pressed */
      while (joytrg(0) == 0);

      if (joytrg(0) & JOY_B) {
         put_string("JOY_B (pos) pushed", 3, 15);
         poke((0x1AE4), shift);

      } else if (joytrg(0) & JOY_A) {
         put_string("JOY_A (neg) pushed", 3, 15);
         poke((0x1AE4), shift_neg);

      } else if (joytrg(0) & JOY_UP) {
         shift++;
         shift_neg--;

      } else if (joytrg(0) & JOY_DOWN) {
         shift--;
         shift_neg++;

      } else if (joytrg(0) & JOY_STRT) {
         init_vars();
         poke(0x1AE3, 0x5A);
         shift = 1;
         shift_neg = 0xff;

      } else if (joytrg(0) & JOY_SLCT) {
         break;

      }
      vsync(1);
   }
}


/*
 * Test 3 is a set of tests to manipulate the memory and indexing
 *        registers
 */

test3()
{
   static char tmp;
   int i,j;

   cls();
   put_string("CDROM Arcade Card Test #3", 5, 2);

   put_string("Initializing Arcade Card...", 3, 27);
   init_test3();

   put_string("<LT>/<RT> = move pointer", 3, 22);
   put_string("<UP>/<DN> = inc/dec item at pointer", 3, 23);
   put_string("<I>       = repeat read", 3, 24);
   put_string("<RUN>     = reinitialize", 3, 26);
   put_string("<SEL> for next test        ", 3, 27);
   disp_test3();

   while(1)
   {
      /* wait until key pressed */
      while (joytrg(0) == 0);

      if (joytrg(0) & JOY_STRT) {
         init_test3();
         disp_test3();
         put_string("<SEL> for next test        ", 3, 24);

      } else if (joytrg(0) & JOY_UP) {
         if (byte < 0x1A07) {
            tmp = peek(byte)+1;
            poke(byte, tmp);
         } else if (byte == 0x1A0A) {
            poke(byte, 1);
         } else {
            tmp = peek(byte) | (1<<bit);
            poke(byte, tmp );
         }
         disp_test3_parta(4);

      } else if (joytrg(0) & JOY_DOWN) {
         if (byte < 0x1A07) {
            tmp = peek(byte)-1;
            poke(byte, tmp);
         } else if (byte == 0x1A0A) {
            poke(byte, 0);
         } else {
            tmp = peek(byte) & ~(1<<bit);
            poke(byte, tmp );
         }
         disp_test3_parta(4);

      } else if (joytrg(0) & JOY_RGHT) {

         if (byte < 0x1A07) {
            byte++;
            if (byte == 0x1A07) {
               bit = 7;
            }
         } else if (byte != 0x1A0A) {
            if (bit == 0) {
               byte++;
               bit = 7;
            } else {
               bit--;
            }
         }
         disp_test3_parta(4);

      } else if (joytrg(0) & JOY_LEFT) {
         if (byte == 0x1A0A) {
            byte--;
            bit = 0;

         } else if (byte >= 0x1A07 && byte <= 0x1A09) {
            if (bit == 7) {
               byte--;
               bit = 0;
            } else {
               bit++;
            }
         } else if (byte > 0x1A02) {
            byte--;
         }
         disp_test3_parta(4);

      } else if (joytrg(0) & JOY_A) {
         disp_test3();

      } else if (joytrg(0) & JOY_SLCT) {
         break;
      }
      vsync(1);
   }
}


disp_test3()
{
   int i,j;

   put_string("Before:", 1, 3);
   disp_test3_parta(4);

   put_string("1A00", 1, 9);
   put_string("Data:", 1, 10);

   for (j = 0; j < 2; j++) {
      for (i = 0; i < 8; i++) {
         put_hex(peek(0x1A00), 2, (i*3)+7, 9+j);
      }
   }

   put_string("1A01", 1, 11);
   put_string("Data:", 1, 12);

   for (j = 0; j < 2; j++) {
      for (i = 0; i < 8; i++) {
         put_hex(peek(0x1A01), 2, (i*3)+7, 11+j);
      }
   }

   put_string("After:", 1, 14);
   disp_test3_parta(15);
}


disp_test3_parta(ypos)
int ypos;
{
   put_string("1A02:", 1, ypos);
   put_hex( peek(0x1A02), 2, 7, ypos);
   put_hex( peek(0x1A03), 2, 10, ypos);
   put_hex( peek(0x1A04), 2, 13, ypos);

   put_string("1A05:", 1, ypos+1);
   put_hex( peek(0x1A05), 2, 7, ypos+1);
   put_hex( peek(0x1A06), 2, 10, ypos+1);

   put_string("1A07:    1A08:    1A09:    1A0A:", 1, ypos+2);
   put_binary( peek(0x1A07),  1, ypos+3);
   put_binary( peek(0x1A08), 10, ypos+3);
   put_binary( peek(0x1A09), 19, ypos+3);

   disp_test3_cursor();
}


disp_test3_cursor()
{
   put_string(" ",   6, 4);
   put_string(" ",   9, 4);
   put_string(" ",  12, 4);
   put_string(" ",   6, 5);
   put_string(" ",   9, 5);
   put_string("        ",  1, 8);
   put_string("        ", 10, 8);
   put_string("        ", 19, 8);
   put_string("    ",  28, 7);

   switch (byte) {
      case 0x1A02:
      case 0x1A03:
      case 0x1A04:
         put_string(">", ((byte-0x1A02) * 3) + 6, 4);
         break;

      case 0x1A05:
      case 0x1A06:
         put_string(">", ((byte-0x1A05) * 3) + 6, 5);
         break;

      case 0x1A07:
      case 0x1A08:
      case 0x1A09:
         put_string("*", ((byte-0x1A07)* 9) + (7-bit) + 1, 8);
         break;

      case 0x1A0A:
         put_string("*", 30, 7);
         break;
   }
/*
   put_hex(byte, 4, 30, 4);
   put_number(bit, 4, 30, 5);
*/
}


read_test3()
{
   int i;

   for (i = 2; i < 0x0B; i++) {
      value[i] = peek(0x1A00 + i);
   }
}


init_test3()
{
   static char tmp;
   int i, j, k;

   poke(0x1A02,0x00);
   poke(0x1A03,0x00);
   poke(0x1A04,0x00);
   poke(0x1A05,0x00);
   poke(0x1A06,0x00);
   poke(0x1A07,0x01);
   poke(0x1A08,0x00);
   poke(0x1A09,0x11);

   for (k = 0; k < 0x2; k++) {
      for (j = 0; j < 0x100; j++) {
         for (i = 0; i < 0x100; i++) {
            poke(0x1A00, (i+j));
         }
      }
   }

   poke(0x1A02,0x00);
   poke(0x1A03,0x00);
   poke(0x1A04,0x00);
   poke(0x1A05,0x00);
   poke(0x1A06,0x00);
   poke(0x1A07,0x01);
   poke(0x1A08,0x00);
   poke(0x1A09,0x00);

   byte = 0x1A02;
   bit = 7;
}

